from perusatproc.util import run_otb_command


def pansharpen(inp, inxs, out):
    base_cmd = 'otbcli_BundleToPerfectSensor -inp {inp} ' \
        '-inxs {inxs} ' \
        '-out "{out}" uint16'
    run_otb_command(base_cmd.format(inp=inp, inxs=inxs, out=out))



# -*- coding: utf-8 -*-
"""
Given a path to a PeruSat-1 product, this script peforms all required steps to
form a single calibrated, orthorectified and pansharpened image.

Final output can be a virtual raster or a tiff file.

"""

import argparse
import sys
import logging

from perusatproc import __version__
from perusatproc import calibration, orthorectification, pansharpening
from perusatproc.util import run_command
from perusatproc.orthorectification import GEOID_PATH, DEM_PATH

import subprocess
import xmltodict
import tempfile
import shutil
import rasterio
import os
from glob import glob
from datetime import datetime

__author__ = "Damián Silvani"
__copyright__ = "Dymaxion Labs"
__license__ = "mit"

_logger = logging.getLogger(__name__)

DEFAULT_TILE_SIZE = 2**14


def process_image(dem_path=None, geoid_path=None, spacing=None, *, src, dst):
    _logger.info(f"Source: {src}")
    _logger.info(f"Destination: {dst}")

    basename = os.path.basename(src)
    name, ext = os.path.splitext(basename)
    dirname = os.path.dirname(src)
    dim_xml = glob(os.path.join(dirname, 'DIM_*.XML'))[0]
    rpc_xml = glob(os.path.join(dirname, 'RPC_*.XML'))[0]

    calibration_dir = os.path.join(dst, '_calib')
    os.makedirs(calibration_dir, exist_ok=True)
    calibration_path = os.path.join(calibration_dir, basename)

    _logger.info("Calibrate %s and write %s", src, calibration_path)
    calibration.calibrate(src_path=src,
                          dst_path=calibration_path,
                          metadata_path=dim_xml)

    rpc_fixed_dir = os.path.join(dst, '_rpc')
    os.makedirs(rpc_fixed_dir, exist_ok=True)
    rpc_fixed_path = os.path.join(rpc_fixed_dir, basename)

    _logger.info("Add RPC tags from %s and write %s", calibration_path,
                 rpc_fixed_path)
    orthorectification.add_rpc_tags(src_path=calibration_path,
                                    dst_path=rpc_fixed_path,
                                    metadata_path=rpc_xml)

    orthorectify_fixed_dir = os.path.join(dst, '_ortho')
    os.makedirs(orthorectify_fixed_dir, exist_ok=True)
    orthorectify_path = os.path.join(orthorectify_fixed_dir, basename)
    _logger.info("Orthorectify %s and write %s", rpc_fixed_path,
                 orthorectify_path)
    orthorectification.orthorectify(src_path=rpc_fixed_path,
                                    dst_path=orthorectify_path,
                                    dem_path=dem_path,
                                    geoid_path=geoid_path,
                                    spacing=spacing)

    _logger.info("Clean up image temporary results")
    shutil.rmtree(calibration_dir)
    shutil.rmtree(rpc_fixed_dir)


def build_virtual_raster(inputs, dst):
    cmd = 'gdalbuildvrt {dst} {inputs}'.format(dst=dst,
                                               inputs=' '.join(inputs))
    run_command(cmd)


def retile_images(src, outdir, tile_size=DEFAULT_TILE_SIZE):
    cmd = 'gdal_retile.py -co TILED=YES ' \
        '-ps {tile_size} {tile_size} ' \
        '-targetDir {outdir} ' \
        '{src}'.format(tile_size=tile_size,
        outdir=outdir,
        src=src)
    run_command(cmd)


def process_product(src,
                    dst,
                    tile_size=DEFAULT_TILE_SIZE,
                    dem_path=None,
                    geoid_path=None,
                    spacing=None,
                    retile=False):
    volumes = glob(os.path.join(src, 'VOL_*'))
    _logger.info("Num. Volumes: {}".format(len(volumes)))

    gdal_imgs = []

    for volume in volumes:
        ms_img = glob(os.path.join(volume, 'IMG_*_MS_*/*.TIF'))[0]
        p_img = glob(os.path.join(volume, 'IMG_*_P_*/*.TIF'))[0]
        process_image(src=ms_img,
                      dst=dst,
                      dem_path=dem_path,
                      geoid_path=geoid_path,
                      spacing=spacing)
        process_image(src=p_img,
                      dst=dst,
                      dem_path=dem_path,
                      geoid_path=geoid_path,
                      spacing=spacing)

        ortho_dir = os.path.join(dst, '_ortho')
        volume_dst_path = os.path.join(dst, '{}.tif'.format(os.path.basename(volume)))
        pansharpening.pansharpen(inp=os.path.join(ortho_dir, os.path.basename(p_img)),
                                 inxs=os.path.join(ortho_dir, os.path.basename(ms_img)),
                                 out=volume_dst_path)
        gdal_imgs.append(volume_dst_path)

        _logger.info("Clean up volume temporary results")
        shutil.rmtree(ortho_dir)

    # Create pansharpened virtual raster
    name, _ = os.path.splitext(os.path.basename(src))
    vrt_path = os.path.join(dst, '{}.vrt'.format(name))
    build_virtual_raster(inputs=gdal_imgs, dst=vrt_path)

    if retile:
        # Retile virtual raster
        tiles_dir = os.path.join(dst, 'tiles')
        _logger.info("Retile %s on %s using size (%d, %d)", vrt_path, tiles_dir, tile_size, tile_size)
        retile_images(src=vrt_path, outdir=tiles_dir, tile_size=tile_size)
        
        # Create virtual raster for all pansharpened tiles
        tile_paths = glob(os.path.join(dst, '*.tif'))
        tiles_vrt_path = os.path.join(tiles_dir, '{}.vrt'.format(name))
        build_virtual_raster(inputs=tile_paths, dst=tiles_vrt_path)
        _logger.info("Create virtual raster %s for tiles", tiles_vrt_path)


def parse_args(args):
    """Parse command line parameters

    Args:
      args ([str]): command line parameters as list of strings

    Returns:
      :obj:`argparse.Namespace`: command line parameters namespace
    """
    parser = argparse.ArgumentParser(
        description=
        "Process a PeruSat-1 product into a set of calibrated, orthorectified and pansharpened tiles",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument("--version",
                        action="version",
                        version="perusatproc {ver}".format(ver=__version__))

    parser.add_argument("-v",
                        "--verbose",
                        dest="loglevel",
                        help="set loglevel to INFO",
                        action="store_const",
                        const=logging.INFO)
    parser.add_argument("-vv",
                        "--very-verbose",
                        dest="loglevel",
                        help="set loglevel to DEBUG",
                        action="store_const",
                        const=logging.DEBUG)

    parser.add_argument("src", help="path to directory containing product")
    parser.add_argument("dst",
                        help="path to output directory containing tiles")

    parser.add_argument("--retile", 
                        dest="retile",
                        action="store_true", 
                        help="Retile processed image")
    parser.add_argument("--no-retile", 
                        dest="retile",
                        action="store_false", 
                        help="Do not retile processed image")
    parser.add_argument("--tile-size",
                        type=int,
                        default=DEFAULT_TILE_SIZE,
                        help="tile size (in pixels)")

    parser.add_argument(
        "--dem",
        help=
        "path to directory containing DEM files (defaults to SRTM 1-arc tiles)"
    )
    parser.add_argument("--geoid",
                        help="path to geoid file (defaults to EGM96 geoid)")
    parser.add_argument("--spacing",
                        default=15,
                        help="resampling grid spacing")

    parser.add_argument("-co",
                        "--create-options",
                        default="",
                        help="GDAL create options")

    return parser.parse_args(args)


def setup_logging(loglevel):
    """Setup basic logging

    Args:
      loglevel (int): minimum loglevel for emitting messages
    """
    logformat = "[%(asctime)s] %(levelname)s:%(name)s:%(message)s"
    logging.basicConfig(level=loglevel,
                        stream=sys.stdout,
                        format=logformat,
                        datefmt="%Y-%m-%d %H:%M:%S")


def main(args):
    """Main entry point allowing external calls

    Args:
      args ([str]): command line parameter list
    """
    args = parse_args(args)
    setup_logging(args.loglevel)

    if not args.geoid:
        _logger.info(f"Using default Geoid: {GEOID_PATH}")
    if not args.dem:
        _logger.info(f"Using default DEM files from: {DEM_PATH}")

    process_product(args.src,
                    args.dst,
                    tile_size=args.tile_size,
                    dem_path=args.dem,
                    geoid_path=args.geoid,
                    spacing=args.spacing,
                    retile=args.retile)


def run():
    """Entry point for console_scripts
    """
    main(sys.argv[1:])


if __name__ == "__main__":
    run()



# -*- coding: utf-8 -*-

import logging
import os
import subprocess

__author__ = "Damián Silvani"
__copyright__ = "Dymaxion Labs"
__license__ = "mit"

_logger = logging.getLogger(__name__)


def run_command(cmd):
    _logger.info(cmd)
    subprocess.run(cmd, shell=True, check=True)


def run_otb_command(cmd):
    otb_env_path = os.getenv("OTB_ENV_PATH")
    if otb_env_path:
        fullcmd = f"{otb_env_path} && {cmd}"
    else:
        fullcmd = cmd
    _logger.info(fullcmd)
    subprocess.run(fullcmd, shell=True, check=True)



# -*- coding: utf-8 -*-

import xmltodict
import logging
import os
from datetime import datetime

__author__ = "Damián Silvani"
__copyright__ = "Dymaxion Labs"
__license__ = "mit"

_logger = logging.getLogger(__name__)

RPC_COEFF_PAIRS = [
    ('samp_num_coeffs', 'COL_NUM_COEFF'),
    ('samp_den_coeffs', 'COL_DEN_COEFF'),
    ('line_num_coeffs', 'ROW_NUM_COEFF'),
    ('line_den_coeffs', 'ROW_DEN_COEFF'),
]


def extract_calibration_metadata(metadata_path):
    with open(metadata_path) as f:
        body = xmltodict.parse(f.read())

    doc = body['Dimap_Document']

    # Image date and time
    strip_source = doc['Dataset_Sources']['Source_Identification'][
        'Strip_Source']
    date_str, time_str = strip_source['IMAGING_DATE'], strip_source[
        'IMAGING_TIME']
    date = datetime.strptime(date_str, "%Y-%m-%d")
    time = datetime.strptime(time_str, "%H:%M:%S")

    geom_values = doc['Geometric_Data']['Use_Area']['Located_Geometric_Values']

    # Sun elevation and azimuth angles
    solar_incidences = geom_values['Solar_Incidences']
    sun_elev = float(solar_incidences['SUN_ELEVATION'])
    sun_azim = float(solar_incidences['SUN_AZIMUTH'])

    # Viewing elevation and azimuth angles
    acquisition_angles = geom_values['Acquisition_Angles']
    view_elev = float(acquisition_angles['VIEWING_ANGLE'])
    view_azim = float(acquisition_angles['AZIMUTH_ANGLE'])

    # Gains and biases for each band
    band_spectral_range = doc['Radiometric_Data']['Radiometric_Calibration'][
        'Instrument_Calibration']['Band_Measurement_List']
    band_radiances = band_spectral_range['Band_Radiance']
    if not isinstance(band_radiances, list):
        band_radiances = [band_radiances]
    gains = [float(r['GAIN']) for r in band_radiances]
    biases = [float(r['BIAS']) for r in band_radiances]

    # Solar irradiance values for each band
    band_solar_irradiances = band_spectral_range['Band_Solar_Irradiance']
    if not isinstance(band_solar_irradiances, list):
        band_solar_irradiances = [band_solar_irradiances]
    solar_irradiances = [float(r['VALUE']) for r in band_solar_irradiances]

    return dict(minute=time.minute,
                hour=time.hour,
                day=date.day,
                month=date.month,
                year=date.year,
                sun_elev=sun_elev,
                sun_azim=sun_azim,
                view_elev=view_elev,
                view_azim=view_azim,
                gains=gains,
                biases=biases,
                solar_irradiances=solar_irradiances)


def extract_projection_metadata(metadata_path):
    with open(metadata_path) as f:
        body = xmltodict.parse(f.read())

    doc = body['Dimap_Document']

    # Raster size
    raster_dimensions = doc['Raster_Data']['Raster_Dimensions']
    sizex = int(raster_dimensions['NCOLS'])
    sizey = int(raster_dimensions['NROWS'])

    # Raster extent
    vertices = doc['Dataset_Content']['Dataset_Extent']['Vertex']
    lats = [float(v['LAT']) for v in vertices]
    lons = [float(v['LON']) for v in vertices]
    minx, maxx = min(lons), max(lons)
    miny, maxy = min(lats), max(lats)

    return dict(sizex=sizex,
                sizey=sizey,
                ulx=minx,
                uly=maxy,
                lrx=maxx,
                lry=miny)


def extract_rpc_metadata(metadata_path):
    with open(metadata_path) as f:
        body = xmltodict.parse(f.read())

    doc = body['Rpc_Document']

    inverse_model = doc['Inverse_Model']
    err_bias = -1.0  # unknown?
    err_rand = -1.0  # unknown?

    coeffs = dict()
    for key, xml_key in RPC_COEFF_PAIRS:
        coeffs[key] = [
            float(inverse_model['{}_{}'.format(xml_key, v)])
            for v in range(1, 21)
        ]

    validity = doc['Validity']
    lon_scale = float(validity['LON_SCALE'])
    lon_offset = float(validity['LON_OFF'])
    lat_scale = float(validity['LAT_SCALE'])
    lat_offset = float(validity['LAT_OFF'])
    height_scale = float(validity['HEIGHT_SCALE'])
    height_offset = float(validity['HEIGHT_OFF'])
    samp_scale = float(validity['COL_SCALE'])
    samp_offset = float(validity['COL_OFF'])
    line_scale = float(validity['ROW_SCALE'])
    line_offset = float(validity['ROW_OFF'])

    return dict(err_bias=err_bias,
                err_rand=err_rand,
                lon_scale=lon_scale,
                lon_offset=lon_offset,
                lat_scale=lat_scale,
                lat_offset=lat_offset,
                height_scale=height_scale,
                height_offset=height_offset,
                samp_scale=samp_scale,
                samp_offset=samp_offset,
                line_scale=line_scale,
                line_offset=line_offset,
                **coeffs)



# -*- coding: utf-8 -*-
"""
This script bundles a pancromatic (P) and multispectral (MS) image to form a
pansharpened image.

"""

from glob import glob
import argparse
import logging
import os
import sys
import tempfile

from perusatproc import __version__
from perusatproc.pansharpening import pansharpen

__author__ = "Damián Silvani"
__copyright__ = "Dymaxion Labs"
__license__ = "mit"

_logger = logging.getLogger(__name__)


def parse_args(args):
    """Parse command line parameters

    Args:
      args ([str]): command line parameters as list of strings

    Returns:
      :obj:`argparse.Namespace`: command line parameters namespace
    """
    parser = argparse.ArgumentParser(
        description=
        "Perform radiometric calibration from Level 2A to Top-of-Atmosphere (ToA)"
    )

    parser.add_argument("--version",
                        action="version",
                        version="perusatproc {ver}".format(ver=__version__))

    parser.add_argument("-v",
                        "--verbose",
                        dest="loglevel",
                        help="set loglevel to INFO",
                        action="store_const",
                        const=logging.INFO)
    parser.add_argument("-vv",
                        "--very-verbose",
                        dest="loglevel",
                        help="set loglevel to DEBUG",
                        action="store_const",
                        const=logging.DEBUG)

    parser.add_argument("src", help="path to input image")
    parser.add_argument("dst", help="path to output image")

    return parser.parse_args(args)


def setup_logging(loglevel):
    """Setup basic logging

    Args:
      loglevel (int): minimum loglevel for emitting messages
    """
    logformat = "[%(asctime)s] %(levelname)s:%(name)s:%(message)s"
    logging.basicConfig(level=loglevel,
                        stream=sys.stdout,
                        format=logformat,
                        datefmt="%Y-%m-%d %H:%M:%S")


def main(args):
    """Main entry point allowing external calls

    Args:
      args ([str]): command line parameter list
    """
    args = parse_args(args)
    setup_logging(args.loglevel)

    _logger.debug("Args: %s", args)

    pansharpen(args.src, args.dst)


def run():
    """Entry point for console_scripts
    """
    main(sys.argv[1:])


if __name__ == "__main__":
    run()



# -*- coding: utf-8 -*-
"""
Adds projection and RPC tags to an image from its metadata files, and them
orthorectifies it using the RPC data and a DEM image.

If no DEM is provided, a DEM from SRTM is used (1-arc second / 30m aprox GSD).

"""

import argparse
import logging
import os
import sys
import shutil

from perusatproc import __version__
from perusatproc.orthorectification import add_rpc_tags, orthorectify, GEOID_PATH, DEM_PATH

__author__ = "Damián Silvani"
__copyright__ = "Dymaxion Labs"
__license__ = "mit"

_logger = logging.getLogger(__name__)


def process_image(rpc_metadata_path=None,
                  dem_path=None,
                  geoid_path=None,
                  spacing=None,
                  *,
                  src_path,
                  dst_path):

    rpc_fixed_dir = os.path.join(os.path.dirname(dst_path), '_rpc')
    os.makedirs(rpc_fixed_dir, exist_ok=True)
    rpc_fixed_path = os.path.join(rpc_fixed_dir, os.path.basename(src_path))

    _logger.info("Add RPC tags from %s and write %s", src_path, rpc_fixed_path)
    add_rpc_tags(src_path=src_path,
                 dst_path=rpc_fixed_path,
                 metadata_path=rpc_metadata_path)

    _logger.info("Orthorectify %s and write %s", rpc_fixed_path, dst_path)
    orthorectify(src_path=rpc_fixed_path,
                 dst_path=dst_path,
                 dem_path=dem_path,
                 geoid_path=geoid_path,
                 spacing=spacing)

    _logger.info("Clean up temporary results")
    shutil.rmtree(rpc_fixed_dir)


def parse_args(args):
    """Parse command line parameters

    Args:
      args ([str]): command line parameters as list of strings

    Returns:
      :obj:`argparse.Namespace`: command line parameters namespace
    """
    parser = argparse.ArgumentParser(
        description=
        "Perform radiometric calibration from Level 2A to Top-of-Atmosphere (ToA)"
    )

    parser.add_argument("--version",
                        action="version",
                        version="perusatproc {ver}".format(ver=__version__))

    parser.add_argument("-v",
                        "--verbose",
                        dest="loglevel",
                        help="set loglevel to INFO",
                        action="store_const",
                        const=logging.INFO)
    parser.add_argument("-vv",
                        "--very-verbose",
                        dest="loglevel",
                        help="set loglevel to DEBUG",
                        action="store_const",
                        const=logging.DEBUG)

    parser.add_argument("src", help="path to input image")
    parser.add_argument("dst", help="path to output image")
    parser.add_argument("-m",
                        "--metadata",
                        required=True,
                        help="path to RPC metadata XML file")
    parser.add_argument(
        "--dem",
        help=
        "path to directory containing DEM files (defaults to SRTM 1-arc tiles)"
    )
    parser.add_argument("--geoid",
                        help="path to geoid file (defaults to EGM96 geoid)")
    parser.add_argument("--spacing",
                        default=15,
                        help="resampling grid spacing")

    return parser.parse_args(args)


def setup_logging(loglevel):
    """Setup basic logging

    Args:
      loglevel (int): minimum loglevel for emitting messages
    """
    logformat = "[%(asctime)s] %(levelname)s:%(name)s:%(message)s"
    logging.basicConfig(level=loglevel,
                        stream=sys.stdout,
                        format=logformat,
                        datefmt="%Y-%m-%d %H:%M:%S")


def main(args):
    """Main entry point allowing external calls

    Args:
      args ([str]): command line parameter list
    """
    args = parse_args(args)
    setup_logging(args.loglevel)

    if not args.geoid:
        _logger.info(f"Using default Geoid: {GEOID_PATH}")
    if not args.dem:
        _logger.info(f"Using default DEM files from: {DEM_PATH}")

    process_image(src_path=args.src,
                  dst_path=args.dst,
                  rpc_metadata_path=args.metadata,
                  dem_path=args.dem,
                  geoid_path=args.geoid,
                  spacing=args.spacing)


def run():
    """Entry point for console_scripts
    """
    main(sys.argv[1:])


if __name__ == "__main__":
    run()



# -*- coding: utf-8 -*-

import logging
import os
import tempfile

from perusatproc.metadata import extract_calibration_metadata
from perusatproc.util import run_otb_command

__author__ = "Damián Silvani"
__copyright__ = "Dymaxion Labs"
__license__ = "mit"

_logger = logging.getLogger(__name__)


def calibrate(*, src_path, dst_path, metadata_path):
    base_cmd = """otbcli_OpticalCalibration \
      -in {src} \
      -out {dst} uint16 \
      -milli true \
      -level toa \
      -acqui.minute {minute} \
      -acqui.hour {hour} \
      -acqui.day {day} \
      -acqui.month {month} \
      -acqui.year {year} \
      -acqui.sun.elev {sun_elev} \
      -acqui.sun.azim {sun_azim} \
      -acqui.view.elev {view_elev} \
      -acqui.view.azim {view_azim} \
      -acqui.gainbias {gainbias_path} float \
      -acqui.solarilluminations {solarillum_path} float
    """
    metadata = extract_calibration_metadata(metadata_path)

    gf = tempfile.NamedTemporaryFile(suffix='.txt', delete=False)
    for k in ('gains', 'biases'):
        line = "{}\n".format(" : ".join(str(v) for v in metadata[k]))
        gf.write(line.encode())
    gf.close()

    sf = tempfile.NamedTemporaryFile(suffix='.txt', delete=False)
    line = "{}\n".format(" : ".join(
        str(v) for v in metadata['solar_irradiances']))
    sf.write(line.encode())
    sf.close()

    cmd = base_cmd.format(src=src_path,
                          dst=dst_path,
                          gainbias_path=gf.name,
                          solarillum_path=sf.name,
                          **metadata)
    run_otb_command(cmd)

    os.unlink(gf.name)
    os.unlink(sf.name)



# -*- coding: utf-8 -*-

import logging
import os
import pkg_resources

import rasterio

from perusatproc.metadata import extract_projection_metadata, extract_rpc_metadata
from perusatproc.util import run_otb_command

__author__ = "Damián Silvani"
__copyright__ = "Dymaxion Labs"
__license__ = "mit"

_logger = logging.getLogger(__name__)

DATA_DIR = pkg_resources.resource_filename('perusatproc', 'data')
GEOID_PATH = os.path.join(DATA_DIR, 'egm96.grd')
DEM_PATH = os.path.join(DATA_DIR, 'dem')

RPC_COEFF_KEYS = [
    'err_bias',
    'err_rand',
    'line_offset',
    'samp_offset',
    'lat_offset',
    'lon_offset',
    'height_offset',
    'line_scale',
    'samp_scale',
    'lat_scale',
    'lon_scale',
    'height_scale',
    'line_num_coeffs',
    'line_den_coeffs',
    'samp_num_coeffs',
    'samp_den_coeffs',
]


def add_rpc_tags(*, src_path, dst_path, metadata_path):
    metadata = extract_rpc_metadata(metadata_path)

    keys = [
        ('ERR_BIAS', 'err_bias'),
        ('ERR_RAND', 'err_rand'),
        ('LINE_OFF', 'line_offset'),
        ('SAMP_OFF', 'samp_offset'),
        ('LAT_OFF', 'lat_offset'),
        ('LONG_OFF', 'lon_offset'),
        ('HEIGHT_OFF', 'height_offset'),
        ('LINE_SCALE', 'line_scale'),
        ('SAMP_SCALE', 'samp_scale'),
        ('LAT_SCALE', 'lat_scale'),
        ('LONG_SCALE', 'lon_scale'),
        ('HEIGHT_SCALE', 'height_scale'),
    ]
    tags = {k1: metadata[k2] for k1, k2 in keys}
    coeffs_keys = [('LINE_NUM_COEFF', 'line_num_coeffs'),
                   ('LINE_DEN_COEFF', 'line_den_coeffs'),
                   ('SAMP_NUM_COEFF', 'samp_num_coeffs'),
                   ('SAMP_DEN_COEFF', 'samp_den_coeffs')]
    for k, v in coeffs_keys:
        tags[k] = ' '.join([str(v2) for v2 in metadata[v]])

    with rasterio.open(src_path) as src:
        with rasterio.open(dst_path, 'w', **src.profile) as dst:
            dst.write(src.read())
            dst.update_tags(ns='RPC', **tags)


def orthorectify(dem_path=None, geoid_path=None, spacing=None, *, src_path, dst_path):
    spacing_opt = ""
    if spacing:
        spacing_opt = "-opt.gridspacing {spacing}".format(spacing=spacing)
    base_cmd = """otbcli_OrthoRectification \
      -io.in \"{src}?&skipcarto=true\" \
      -io.out {dst} uint16 \
      -outputs.mode auto \
      -elev.geoid {geoid_path} \
      -elev.dem {dem_path} \
      {spacing_opt}
    """

    if not geoid_path:
        geoid_path = GEOID_PATH
    if not dem_path:
        dem_path = DEM_PATH

    cmd = base_cmd.format(src=src_path,
                          dst=dst_path,
                          geoid_path=geoid_path,
                          dem_path=dem_path,
                          spacing_opt=spacing_opt)
    run_otb_command(cmd)
